package me.mrletsplay.versionspawns;

import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.ArrayList;
import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import me.mrletsplay.mrcore.json.JSONObject;
import me.mrletsplay.versionspawns.command.DelWarpCommand;
import me.mrletsplay.versionspawns.command.SetWarpCommand;
import me.mrletsplay.versionspawns.command.WarpCommand;
import net.fabricmc.api.ModInitializer;
import net.fabricmc.fabric.api.command.v2.CommandRegistrationCallback;
import net.fabricmc.fabric.api.event.lifecycle.v1.ServerLifecycleEvents;
import net.minecraft.registry.RegistryKey;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.server.MinecraftServer;
import net.minecraft.server.world.ServerWorld;
import net.minecraft.util.Identifier;

public class VersionSpawnsMod implements ModInitializer {

	public static final Logger LOGGER = LoggerFactory.getLogger("versionspawns");

	private static final Path WARPS_FILE = Paths.get("config/versionspawns/warps.json");

	public static MinecraftServer server;

	private static JSONObject warps;

	@Override
	public void onInitialize() {
		try {
			Files.createDirectories(WARPS_FILE.getParent());
		} catch (IOException e) {
			LOGGER.error("Failed to create config directory", e);
			return;
		}

		if(loadWarps() == null) {
			LOGGER.error("Failed to load warps, exiting");
			return;
		}

		CommandRegistrationCallback.EVENT.register((dispatcher, registryAccess, environment) -> {
			dispatcher.register(new WarpCommand().create());
			dispatcher.register(new SetWarpCommand().create());
			dispatcher.register(new DelWarpCommand().create());
		});

		ServerLifecycleEvents.SERVER_STARTED.register(s -> server = s);

		LOGGER.info("VersionSpawns loaded!");
	}

	public static List<String> getWarps() {
		return new ArrayList<>(warps.keys());
	}

	private static JSONObject loadWarps() {
		if(warps != null) return warps;

		try {
			if(!Files.exists(WARPS_FILE)) {
				Files.createFile(WARPS_FILE);
			}

			String str = Files.readString(WARPS_FILE, StandardCharsets.UTF_8);
			if(str.isEmpty()) str = "{}";
			return warps = new JSONObject(str);
		} catch (IOException e) {
			LOGGER.error("Failed to read config", e);
			return null;
		}
	}

	private static void saveSpawns(JSONObject warps) {
		try {
			VersionSpawnsMod.warps = warps;
			Files.write(WARPS_FILE, warps.toFancyString().getBytes(StandardCharsets.UTF_8));
		} catch (IOException e) {
			LOGGER.error("Failed to write config", e);
			return;
		}
	}

	public static void setWarp(String name, Location loc) {
		JSONObject spawns = loadWarps();
		spawns.set(name, loc.toJSON());
		saveSpawns(spawns);
	}

	public static Location getWarp(String name) {
		JSONObject warp = loadWarps().optJSONObject(name).orElse(null);
		if(warp == null) return null;
		return Location.fromJSON(warp);
	}

	public static void deleteWarp(String name) {
		JSONObject spawns = loadWarps();
		spawns.unset(name);
		saveSpawns(spawns);
	}

	public static String getWorldIdentifier(ServerWorld world) {
		return world.getRegistryKey().getValue().getNamespace() + ":" + world.getRegistryKey().getValue().getPath();
	}

	public static ServerWorld getWorldByIdentifier(String identifier) {
		String[] spl = identifier.split(":", 2);
		if(spl.length != 2) throw new IllegalArgumentException("Invalid identifier");
		return server.getWorld(RegistryKey.of(RegistryKeys.WORLD, Identifier.of(spl[0], spl[1])));
	}

}